using Microsoft.AspNetCore.RateLimiting;

var builder = WebApplication.CreateBuilder(args);
var app = builder.Build();

// Configure base rate limiter options
var limiterOptions = new RateLimiterOptions {
    OnRejected = async (context, cancellationToken) => {
        // Set HTTP error 429: Too many requests
        context.HttpContext.Response.StatusCode = 429;

        // Add Retry-After header, if possible
        if (context.Lease.TryGetMetadata("RETRY_AFTER", out var value) && value is TimeSpan retryAfter) {
            context.HttpContext.Response.Headers.RetryAfter = retryAfter.TotalSeconds.ToString();
        }

        // Send text response
        await context.HttpContext.Response.WriteAsync("Request rejected. Try again later.", cancellationToken);
    }
};

// Add limiting policies
limiterOptions.AddTokenBucketLimiter("token-bucket", options => {
    options.TokenLimit = 5;
    options.ReplenishmentPeriod = TimeSpan.FromSeconds(10);
    options.TokensPerPeriod = 1;
    options.AutoReplenishment = true;
    options.QueueLimit = 1;
});
limiterOptions.AddFixedWindowLimiter("fixed-window", options => {
    options.Window = TimeSpan.FromSeconds(10);
    options.PermitLimit = 5;
    options.AutoReplenishment = true;
    options.QueueLimit = 0;
});

// Enable rate limiter middleware
app.UseRateLimiter(limiterOptions);

// Add rate limited endpoints
app.MapGet("/token-bucket", () => "Hello World!").RequireRateLimiting("token-bucket");
app.MapGet("/fixed-window", () => "Hello World!").RequireRateLimiting("fixed-window");

app.Run();
